#include <built_in.h>
char            filename[14] = "sound1.mp3";           // Set File name
unsigned long   i, file_size;
const           BUFFER_SIZE = 512;
char            data_buffer_32[32], BufferLarge[BUFFER_SIZE];
sbit            Mmc_Chip_Select at RC0_bit;
sbit            Mmc_Chip_Select_Direction at TRISC0_bit;
void SW_SPI_Write(char data_) {              // Writes one byte to MP3 SDI
  RD1_bit = 1;                               // Set BSYNC before sending the first bit
  RD2_bit = 0;  RD3_bit = data_;  RD2_bit = 1;  data_ >>= 1;  // Send data_ LSB, data_.0
  RD2_bit = 0;  RD3_bit = data_;  RD2_bit = 1;  data_ >>= 1;  // Send data_.1
  RD1_bit = 0;                               // Clear BSYNC after sending the second bit
  RD2_bit = 0;  RD3_bit = data_;  RD2_bit = 1;  data_ >>= 1;  // Send data_.2
  RD2_bit = 0;  RD3_bit = data_;  RD2_bit = 1;  data_ >>= 1;  // Send data_.3
  RD2_bit = 0;  RD3_bit = data_;  RD2_bit = 1;  data_ >>= 1;  // Send data_.4
  RD2_bit = 0;  RD3_bit = data_;  RD2_bit = 1;  data_ >>= 1;  // Send data_.5
  RD2_bit = 0;  RD3_bit = data_;  RD2_bit = 1;  data_ >>= 1;  // Send data_.6
  RD2_bit = 0;  RD3_bit = data_;  RD2_bit = 1;  data_ >>= 1;  // Send data_.7
  RD2_bit = 0;
}
void MP3_SCI_Write(char address, unsigned int data_in) { // Writes one word to MP3 SCI
  RC1_bit = 0;                              // select MP3 SCI
  SPI1_Write(0x02);                         // send WRITE command
  SPI1_Write(address);
  SPI1_Write(Hi(data_in));                  // Send High byte
  SPI1_Write(Lo(data_in));                  // Send Low byte
  RC1_bit = 1;                              // deselect MP3 SCI
  while (RD0_bit == 0);                     // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
}
// Reads words_count words from MP3 SCI
void MP3_SCI_Read(char start_address, char words_count, unsigned int *data_buffer) {
  unsigned int temp;
  RC1_bit = 0;                              // select MP3 SCI
  SPI1_Write(0x03);                         // send READ command
  SPI1_Write(start_address);
  while (words_count--) {                   // read words_count words byte per byte
    temp = SPI1_Read(0);
    temp <<= 8;
    temp += SPI1_Read(0);
    *(data_buffer++) = temp;
  }
  RC1_bit = 1;                              // deselect MP3 SCI
  while (RD0_bit == 0);                     // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
}
void MP3_SDI_Write(char data_) {            // Write one byte to MP3 SDI
  while (RD0_bit == 0) ;                    // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
  SW_SPI_Write(data_);
}
void MP3_SDI_Write_32(char *data_) {        // Write 32 bytes to MP3 SDI
  char i;
  while (RD0_bit == 0) ;                    // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
  for (i=0; i<32; i++) SW_SPI_Write(data_[i]);
}
void Set_Clock(unsigned int clock_khz, char doubler) {   // Set clock
  clock_khz /= 2;                           // calculate value
  if (doubler) clock_khz |= 0x8000;
  MP3_SCI_Write(0x03, clock_khz);           // Write value to CLOCKF register
}
void Init() {
  ADCON1 |= 0x0F;                           // set all AN pins to digital
  RD2_bit = 0;    RD3_bit = 0;              // Clear SW SPI SCK and SDO
  TRISD2_bit = 0; TRISD3_bit = 0;           // Set SW SPI pin directions
  RC1_bit = 1;                              // Deselect MP3_CS
  TRISC1_bit = 0;                           // Configure MP3_CS as output
  RC2_bit = 1;                              // Set MP3_RST pin
  TRISC2_bit = 0;                           // Configure MP3_RST as output
  TRISD0_bit = 1;                           // Configure DREQ as input
  RD1_bit = 0;                              // Clear BSYNC
  TRISD1_bit = 0;                           // Configure BSYNC as output
}
// Software Reset
void Soft_Reset() {
  MP3_SCI_Write(0x00,0x0204);               // Write to MODE register
                                            //   set SM_RESET bit and SM_BITORD bit(bitorder is LSB first)
  Delay_us(2);                              // Required, see MP3 codec datasheet -> Software Reset
  while (RD0_bit == 0);                     // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
  for (i=0; i<2048; i++) MP3_SDI_Write(0);  // feed 2048 zeros to the MP3 SDI bus
}
void main() {                               // main function
  Init();
  SPI1_Init_Advanced(_SPI_MASTER_OSC_DIV64, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_LOW, _SPI_LOW_2_HIGH);
  Set_Clock(25000,0);                       // Set clock to 25MHz, do not use clock doubler
  Soft_Reset();                             // SW Reset
  if (Mmc_Fat_Init() == 0) {
    if (Mmc_Fat_Assign(&filename, 0) ) {    // Assign file "sound1.mp3"
      Mmc_Fat_Reset(&file_size);            // Call Reset before file reading
      while (file_size > BUFFER_SIZE) {     // Send file blocks to MP3 SDI
        for (i=0; i<BUFFER_SIZE; i++)       // Read file block
          Mmc_Fat_Read(BufferLarge + i);
        for (i=0; i<BUFFER_SIZE/32; i++)    // Send file block to mp3 decoder
          MP3_SDI_Write_32(BufferLarge + i*32);
        file_size -= BUFFER_SIZE;           // Decrease file size
      }
      for (i=0; i<file_size; i++)           // Send the rest of the file
        Mmc_Fat_Read(BufferLarge + i);
      for (i=0; i<file_size; i++)
        MP3_SDI_Write(BufferLarge[i]);
    }
  }
}
